package cse425s.hw2.syntax;
import cse425s.hw2.syntax.statement.*;
import cse425s.hw2.syntax.token.*;
import java.io.IOException;
import java.util.Collection;
import java.util.LinkedList;

public class DefaultParser implements Parser
{
    public DefaultParser(Lexer lexer){
        _lexer = lexer;
        _stmt  = null;
    }
    
    public Statement peek() throws IOException{
        while ( _stmt == null ){
            try{
                if ( _lexer.has("define") ){
                    _stmt = parseDefineStatement();
                }else if ( _lexer.has("grant") ){
                    _stmt = parseGrantStatement();
                }else if ( _lexer.has("check") ){
                    _stmt = parseCheckStatement();
                }else{
                    Token nexttoken = _lexer.read();
                    System.err.println("[ERROR] In \"" + nexttoken.getFileName() + "\" at line "+nexttoken.getLineNumber() + " at offset " + nexttoken.getLineOffset() + ":  Expected \"define\", \"grant\", or \"check\"; found \"" + nexttoken.getLexeme() + "\".");
                }
            }catch(java.util.regex.PatternSyntaxException invalidstatement){
                _lexer.read();
            }
        }
        return _stmt;
    }
    
    public Statement read() throws IOException{
        Statement result=peek();
        _stmt = null;
        return result;
    }
    
    private DefineStatement parseDefineStatement() throws IOException{
        _lexer.match("define");
        String classname=((Identifier) (_lexer.match(Identifier.class))).toString();
        Collection<String> parents = null;
        if ( _lexer.has("inherits") ){
            _lexer.read();
            parents = parseIdentifierList();
        }else{
            parents = new LinkedList<String>();
        }
        
        Collection<String> passwds = null;
        if ( _lexer.has("passwords") ){
            _lexer.read();
            passwds = parseIdentifierList();
        }else{
            passwds = new LinkedList<String>();
        }
        
        Collection<String> capabilities = null;
        if ( _lexer.has("capabilities") ){
            _lexer.read();
            capabilities = parseIdentifierList();
        }else{
            capabilities = new LinkedList<String>();
        }
        
        if ( _lexer.has("define") || _lexer.has("grant") || _lexer.has("check") ){
            System.err.println("[ERROR] In \""+_lexer.peek().getFileName()+"\" at line "+_lexer.peek().getLineNumber()+" at offset "+_lexer.peek().getLineOffset()+": Missing semicolon after define statment.");
        }else{
            _lexer.match(";");
        }
        return new DefineStatement(classname,parents,passwds,capabilities);
    }
    
    private Collection<String> parseIdentifierList() throws IOException {
        Collection<String> result = new LinkedList<String>();
        result.add(_lexer.match(Identifier.class).getLexeme());
        while ( _lexer.has(",") ){
            _lexer.match(",");
            result.add(_lexer.match(Identifier.class).getLexeme());
        }
        return result;
    }
    
    private GrantStatement parseGrantStatement() throws IOException {
        _lexer.match("grant");
        String classname=_lexer.match(Identifier.class).toString();
        _lexer.match("to");
        String instname=_lexer.match(Identifier.class).toString();
        if ( _lexer.has(";") ){
            _lexer.read();
        }else if ( _lexer.has("define") || _lexer.has("grant") || _lexer.has("check") ){
            System.err.println("[ERROR] In \""+_lexer.peek().getFileName()+"\" at line "+_lexer.peek().getLineNumber()+" at offset "+_lexer.peek().getLineOffset()+": Missing semicolon after grant statment.");
        }
        return new GrantStatement(classname,instname);
    }
    
    private CheckStatement parseCheckStatement() throws IOException {
        _lexer.match("check");
        String instname=_lexer.match(Identifier.class).toString();
        if ( _lexer.has(";") ){
            _lexer.read();
        }else if ( _lexer.has("define") || _lexer.has("grant") || _lexer.has("check") ){
            System.err.println("[ERROR] In \""+_lexer.peek().getFileName()+"\" at line "+_lexer.peek().getLineNumber()+" at offset "+_lexer.peek().getLineOffset()+": Missing semicolon after check statment.");
        }
        return new CheckStatement(instname);
    }
    
    private Lexer _lexer;
    private Statement _stmt;
}
